/**
 * @packageDocumentation
 * @module shared-drive-extension
 */
import { JupyterFrontEndPlugin } from '@jupyterlab/application';
/**
 * Export the plugins as default.
 */
declare const plugins: JupyterFrontEndPlugin<any>[];
export default plugins;
